﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;
using System.IO;
using System.Diagnostics;

using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

namespace RCAddInSelectLayerGroupCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        // 設定値
        public AddInSettings addInSettings = null;

        // このアドインのユニーク名
        public string myAddInUniqueName = "RCAddInSelectLayerGroupDockingBar";

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            // 多言語対応リソースのDLLファイルをロード
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            // Have you already started it ?
            bool alreadyStartedIt = false;
            CustomDockingBar dockingBar = DockingBars.Find(myAddInUniqueName);
            if(dockingBar != null)
                alreadyStartedIt = true;

            // アドインのウィンドウのタイトル
            string dockingBarName = Properties.Resources.addInProgramName;

            // アドインの設定値をファイルからを読み込む
            if(!LoadAddInSettings(out addInSettings))
                addInSettings = new AddInSettings();// 読み込めなかった場合は初期値

            // このアドインを RootPro CAD にドッキングバーとして追加
            // ドッキングバーのサイズ初期値・スタイルを指定
            // ※ウィンドウの配色と表示フォントの設定は、RootPro CAD 側で Off の場合は各アドインでの指定は無効になる
            dockingBar = DockingBars.Add(myAddInUniqueName, dockingBarName, 460, 860, addInSettings.rootProCADVisualStyle, addInSettings.rootProCADDisplayFont);

            // アドインのウィンドウのタイトルはレジストリに保存されているので異なる場合はここで変える
            if(dockingBar.Text != dockingBarName)
                dockingBar.Text = dockingBarName;
            
            dockingBar.UserCtrl = new RootProCADUserControl(this);// プロジェクトで「追加」-「ユーザーコントロール（Windows フォーム）」で追加した「RootPro CAD フォームコントロール」

            // Adjust FloatingBar
            if(!alreadyStartedIt)
                dockingBar.AdjustFloatingBar();

            // フォームが表示されていない場合に表示されるためのコマンドを追加
            // RootPro CAD の「アドイン」メニューにコマンドが表示される（これをしなくても「表示」の「ユーザー定義ウィンドウ」から表示されられるが）
            string commandName = dockingBarName;
            CommandManager.AddMacroCommand(dockingBarName, commandName, MacroCommand);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            // 追加したコマンドを削除
            CommandManager.RemoveMacroCommand(MacroCommand);
        }

        /// <summary>
        /// 外部フォルダのアセンブリを動的にロード
        /// 多言語対応リソースのDLLファイルをロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
        {
            AssemblyName assemblyName = new AssemblyName(args.Name);

            foreach(Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if(AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
                    return assembly;
            }

            if(assemblyName.Name.EndsWith(".resources"))
            {
                if(assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    string dllPath = Path.GetDirectoryName(this.AddInFullPath);
                    dllPath = Path.Combine(dllPath, assemblyName.CultureName);
                    dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

                    if(File.Exists(dllPath))
                    {
                        try
                        {
                            return Assembly.LoadFrom(dllPath);
                        }
                        catch
                        {
                        }
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// マクロコマンドの実行
        /// </summary>
        private void MacroCommand()
        {
            // このアドインのドッキングバーをアクティブにする
            ActivateMyAddInDockingBar();
        }

        /// <summary>
        /// このアドインのドッキングバーをアクティブにする
        /// </summary>
        public void ActivateMyAddInDockingBar()
        {
            var dockingBar = DockingBars.Find(myAddInUniqueName);
            if(dockingBar != null)
                dockingBar.Activate();
        }

        /// <summary>
        /// Get my dockingBar
        /// </summary>
        /// <returns></returns>
        public CustomDockingBar GetMyDockingBar()
        {
            DockingBarCollection dockingBars = DockingBars;
            if(dockingBars == null)
                return null;

            CustomDockingBar myDockingBar = dockingBars.Find(myAddInUniqueName);
            Debug.Assert(myDockingBar != null);
            return myDockingBar;
        }


        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion
    }
}
