﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Geometry;

namespace RCAddInSelectLayerGroupCS
{
    public partial class DrawinglayoutSettingForm:Form
    {

        private LayerGroupsLayerControl parentControl = null;

        public class ItemSet
        {
            public String ItemDisp{ get; set; }
            public Layout  ItemValue{ get; set; }

            // プロパティをコンストラクタでセット
            public ItemSet(String s, Layout v)
            {
                ItemDisp  = s;
                ItemValue = v;
            }
        }


        public DrawinglayoutSettingForm(LayerGroupsLayerControl parentControl)
        {
            this.parentControl = parentControl;

            InitializeComponent();
        }
         
        /// <summary>
        /// 部分図レイアウト情報のセット
        /// </summary>
        /// <param name="doc"></param>
        /// <param name="selectLayoutID"></param>
        public void SetDrawingLayouts(DrawingCollection drawings, int selectLayoutID)
        {
            Debug.Assert(drawings != null);

            int selectIndex = 0;

            List<ItemSet> src = new List<ItemSet>();

            // 部分図レイアウトを順番にセット
            src.Add(new ItemSet("なし", null));// 先頭に「なし」を追加
            int listCount = 1;
            foreach(Drawing drawing in drawings)
            {
                // 用紙上にある部分図レイアウト
                if(drawing is Paper)
                {
                    // 部分図レイアウト
                    LayoutCollection layouts = drawing.Layouts;
                    if(layouts.Count == 0)
                        continue;

                    foreach(Layout layout in layouts)
                    {
                        // 用紙自身のはスキップ
                        if(layout.Drawing is Paper)
                            continue;

                        src.Add(new ItemSet(layout.Name, layout));
                        listCount++;

                        // 指定されたIDと同じIDの部分図レイアウトならこれを選択状態にする
                        if(layout.ID == selectLayoutID)
                            selectIndex = listCount - 1;
                    }
                }
            }

            // ComboBoxに表示と値をセット
            if(src.Count > 0)
            {
                selectDrawinglayoutComboBox.DataSource = src;
                selectDrawinglayoutComboBox.DisplayMember = "ItemDisp";
                selectDrawinglayoutComboBox.ValueMember = "ItemValue";
            }

            // 選択
            selectDrawinglayoutComboBox.SelectedIndex = selectIndex;
        }

        /// <summary>
        /// 部分図レイアウトコンボボックスの選択切り換え時のイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SelectDrawinglayoutComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            string scaleNumeratorText = "";
            string scaleDenominatorText = "";
            bool enableFlg = false;

            // 選択された内容取得
            ItemSet item = ((ItemSet)selectDrawinglayoutComboBox.SelectedItem);
            Layout layout  = item.ItemValue;
            if(layout != null)
            {
                // スケール
                double scaleXNumerator = layout.ScaleXNumerator;      // 分子値
                double scaleXDenominator = layout.ScaleXDenominator;  // 分母値
                scaleNumeratorText = scaleXNumerator.ToString();
                scaleDenominatorText = scaleXDenominator.ToString();

                enableFlg = true;
            }

            scaleNumeratorXComboBox.Enabled = enableFlg;
            scaleDenominatorXComboBox.Enabled = enableFlg;

            // 分子値コンボに値セット
            //scaleNumeratorXComboBox.SelectedIndex = 0;
            scaleNumeratorXComboBox.Text = scaleNumeratorText;
            // 分母値コンボに値セット
            //scaleDenominatorXComboBox.SelectedIndex = 0;
            scaleDenominatorXComboBox.Text = scaleDenominatorText;

           
        }

        /// <summary>
        /// 選択されている部分図レイアウトを取得
        /// </summary>
        /// <returns></returns>
        public Layout GetSelectedLayout()
        {
            ItemSet item = ((ItemSet)selectDrawinglayoutComboBox.SelectedItem);
            Layout layout = item.ItemValue;
            if(layout != null)
            {
                return layout;
            }

            return null;
        }

        /// <summary>
        /// スケール分子値
        /// </summary>
        /// <param name="scaleNumerator"></param>
        /// <returns></returns>
        public bool GetScaleNumerator(out double scaleNumerator)
        {
            scaleNumerator = 1.0;

            string scaleNumeratorText = scaleNumeratorXComboBox.Text;
            bool result = double.TryParse(scaleNumeratorText, out scaleNumerator);

            return result;
        }

        /// <summary>
        /// スケール分母値
        /// </summary>
        /// <param name="scaleDenominator"></param>
        /// <returns></returns>
        public bool GetScaleDenominator(out double scaleDenominator)
        {
            scaleDenominator = -1.0;

            string scaleDenominatorText = scaleDenominatorXComboBox.Text;
            bool result = double.TryParse(scaleDenominatorText, out scaleDenominator);

            return result;
        }

        /// <summary>
        /// OK ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OkButton_Click(object sender, EventArgs e)
        {
            ItemSet item = ((ItemSet)selectDrawinglayoutComboBox.SelectedItem);
            Layout layout = item.ItemValue;
            if(layout != null)
            {
                string msgTitle = Properties.Resources.addInProgramName;

                // スケール値チェック
                // 分子
                double scaleNumerator = 1;
                bool result1 = GetScaleNumerator(out scaleNumerator);
                if(result1 == false || scaleNumerator < Precision.Confusion)
                {
                    string msg = Properties.Resources.scaleSettingMsg_1;// "スケールの分子が不正です。";
                    MessageBox.Show(msg, msgTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    scaleNumeratorXComboBox.Focus();
                    return;
                }
                // 分母
                double scaleDenominator = 1;
                bool result2 = GetScaleDenominator(out scaleDenominator);
                if(result2 == false || scaleDenominator < Precision.Confusion)
                {
                    string msg = Properties.Resources.scaleSettingMsg_2;//"スケールの分母が不正です。";
                    MessageBox.Show(msg, msgTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    scaleDenominatorXComboBox.Focus();
                    return;
                }

                int selectedLayoutID = layout.ID;
                bool stat = parentControl.IsInterlockingDrawinglayoutAtOtherLayerGroup(selectedLayoutID);
                if(stat)
                {
                    string layerGroupName = parentControl.GetLayerGroupName();
                    string layoutName = layout.Name;
                    //string msg = "指定した部分図レイアウト(" + layoutName + ")は\n他のレイヤグループでも連携させるようになっています。\nレイヤグループ(" + layerGroupName + ")でも連動させますか?";
                    //DialogResult result = MessageBox.Show(msg, msgTitle, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
                    //string msg = "指定した部分図レイアウト" + "(" + layoutName + ")" + "は\n他のレイヤグループでも連携させるようになっているため選ぶことができません。";
                    string msg = String.Format(Properties.Resources.setInterlockingDrawinglayoutMsg, layoutName);
                    DialogResult result = MessageBox.Show(msg, msgTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //if(result == DialogResult.Cancel)
                        return;
                }

                // ドキュメントの変更はBeginUndoUnitするところで
                //// XY 異スケールになしない
                //layout.EnableScaleY = false;
                //// スケール値セット
                //layout.ScaleXNumerator = scaleNumerator;
                //layout.ScaleXDenominator = scaleDenominator;
            }

            this.DialogResult = DialogResult.OK;

            this.Close();
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;

            this.Close();
        }

        private void selectDrawinglayoutComboBox_DropDown(object sender, EventArgs e)
        {
            // 現在UIでコンボ自体を無効(Enabled = false)にしている。
            Debug.Assert(false);// 部分図レイアウトとレイヤの関係が複雑になるため、現在は部分図レイアウトを切り替えられなくしている。 
        }
    }
}
