﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;

using RootPro.RootProCAD;

namespace RCAddInSelectLayerGroupCS
{
    public partial class EditLayerNameForm:Form
    {
        Document doc = null;
        Layer layer = null;

        public EditLayerNameForm(Document doc, Layer layer)
        {
            this.doc = doc;
            this.layer = layer;

            InitializeComponent();
        }
        
        private void EditLayerNameForm_Load(object sender, EventArgs e)
        {
            if(layer.IsGroup)
            {
                nameLabel.Text = Properties.Resources.layerGroupName + ":";// "レイヤグループ名:";
                this.Text = Properties.Resources.renameLayerGroup + ":(" + layer.Name + ")";// "レイヤグループ名の変更:(" + layer.Name + ")";
            }
            else
            {
                nameLabel.Text = Properties.Resources.layerName + ":";// "レイヤ名:";
                this.Text = Properties.Resources.renameLayer + ":(" + layer.Name + ")";// "レイヤ名の変更:(" + layer.Name + ")";
            }

            nameTextBox.Text = layer.Name;
        }

        public string GetName()
        {
            string name = nameTextBox.Text;

            return name;
        }

        /// <summary>
        /// 同じ名前のレイヤ又はレイヤグループがないか
        /// </summary>
        /// <param name="parentID"></param>
        /// <param name="layerName"></param>
        /// <param name="myLayer"></param>
        /// <returns></returns>
        private bool IsExistLayerName(int parentID, string layerName, Layer myLayer)
        {
            Debug.Assert(this.layer != null);

            LayerTable layerTable = doc.LayerTable;
            Layer layerGroup = layerTable.GetLayerByID(parentID);
            Debug.Assert(layerGroup.IsGroup);
            LayerCollection layerCollection = layerGroup.ChildLayers;

            foreach(Layer chaildLayer in layerCollection)
            {
		        // 自身は無視する
		        if(chaildLayer.ID == myLayer.ID)
			        continue;

                // myLayerがグループではない場合グループは無視する
                if(!myLayer.IsGroup && chaildLayer.IsGroup)
                    continue;
                // myLayerがグループの場合グループでないレイヤは無視する
                else if(myLayer.IsGroup && !chaildLayer.IsGroup)
                    continue;

                // 大文字と小文字を区別しないで文字列を比較
                if(String.Compare(layerName, chaildLayer.Name, true) == 0)
			        return true;
	        }

	        return false;
        }

        /// <summary>
        /// OK ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            Debug.Assert(this.layer != null);

            // 同じ名前のレイヤグループやレイヤがないか名前チェック
            string name = nameTextBox.Text.Trim();// 前後のスペースは削除
            bool err = false;
            if(name == "")
            {
                err = true;
            }
            else
            {
                int parentID = this.layer.ParentLayer.ID;
                bool isExistLayerName = IsExistLayerName(parentID, name, this.layer);
                if(isExistLayerName == true)
                    err = true;
            }

            if(err)
            {
                
                string strErr = Properties.Resources.renameGroupMsg;// 指定された名前が同じグループで既に使われているか名前が不正です。
                MessageBox.Show(strErr, Properties.Resources.addInProgramName, MessageBoxButtons.OK, MessageBoxIcon.Error);

                nameTextBox.Focus();
                nameTextBox.SelectAll();
                return;
            }

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

    }
}
