﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;

using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;


namespace RCAddInSelectLayerGroupCS
{
    /// <summary>
    /// レイヤグループ＆レイヤ TabPage コントロール
    /// </summary>
    public partial class LayerGroupsLayerControl:UserControl
    {
        private RadioButton[] currentLayerButtons;// カレントボタンの配列
        private LayerStateButton[] layerStateButtons;// レイヤ状態ボタンの配列
        private Label[] layerNameLabels;// レイヤ名ラベルの配列
        private Label[] layerNumberLabels;// レイヤ番号ラベルの配列(コンパクト表示のとき用)

        private RootProCADUserControl rcAddinControl = null;

        public int layerGroupsLayerControlIndex = -1;
        Document doc = null;
        Layer layerGroup = null;

        // ツールチップに表示させる文字列
        string layerGroupNameLabelToolTipHeadText = Properties.Resources.layerGroupNameLabelToolTipHeadText;// "レイヤグループ名:";
        string drawinglayoutSettingButtonToolTipText = Properties.Resources.drawinglayoutSettingButtonToolTipText;// "レイヤグループと連携する部分図レイアウトのスケールを設定できます";
        string interlockingDrawingLayoutNameLabelToolTipText = Properties.Resources.interlockingDrawingLayoutNameLabelToolTipText;// "レイヤグループと連携している部分図レイアウト名";
        string scaleLabelToolTipText = Properties.Resources.scaleLabelToolTipText;// "レイヤグループと連携している部分図レイアウトのスケール";
        string currentLayerButtonsToolTipText = Properties.Resources.currentLayerButtonsToolTipText;// "クリックするとカレントレイヤにします";
        string layerStateButtonsToolTipText = Properties.Resources.layerStateButtonsToolTipText;// "クリックするとレイヤの状態を変えられます";
        string layerNameLabelsToolTipHeadText = Properties.Resources.layerNameLabelsToolTipHeadText;// "レイヤ名:";

        // メッセージ文字列
        string interlockingDrawinglayoutSetting_InformationMessage1 = Properties.Resources.interlockingDrawinglayoutSetting_InformationMsg_1;// "連携する部分図レイアウトを「なし」にしました。カレントの部分図レイアウトは「用紙」なっています。\n\n作図する場合は、連携する部分図レイアウトを設定し直すか、\n連携する部分図レイアウトを設定している他のグループを選択してからにしてください。";

        // コンテキストメニューを開いたとき選んだレイヤのインデックス
        private int lastContextMenuSelectLayerIndex = -1;

        public LayerGroupsLayerControl(RootProCADUserControl parentControl)
        {
            rcAddinControl = parentControl;

            InitializeComponent();

            // レイヤボタンの配列を用意して文字をセット
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;

            // カレントボタンの配列作成
            currentLayerButtons = new RadioButton[layerSize];
            for(int i = 0; i < layerSize; i++)
            {
                string controlName = "currentLayerRadioButton" + i.ToString();
                RadioButton radioButton = (RadioButton)AddInTools.FindControl(this, controlName);
                if(radioButton != null)
                    currentLayerButtons[i] = radioButton;
            }

            // レイヤ状態ボタンの配列作成
            layerStateButtons = new LayerStateButton[layerSize];
            for(int i = 0; i < layerSize; i++)
            {
                string controlName = "layerStateButton" + i.ToString();
                LayerStateButton layerStateButton = (LayerStateButton)AddInTools.FindControl(this, controlName);
                if(layerStateButton != null)
                    layerStateButtons[i] = layerStateButton;
            }

            // レイヤ名ラベルの配列作成
            layerNameLabels = new Label[layerSize];
            for(int i = 0; i < layerSize; i++)
            {
                string controlName = "layerNameLabel" + i.ToString();
                Label label = (Label)AddInTools.FindControl(this, controlName);
                if(label != null)
                    layerNameLabels[i] = label;
            }

            // レイヤ番号ラベル(コンパクト表示のとき用)の配列作成
            layerNumberLabels = new  Label[layerSize];
            for(int i = 0; i < layerSize; i++)
            {
                string controlName = "layerNumberLabel" + i.ToString();
                Label label = (Label)AddInTools.FindControl(this, controlName);
                if(label != null)
                    layerNumberLabels[i] = label;
            }

            // コントロールの位置と表示を整える
            AlignmentAndVisibleControls();

            // ツールチップ、コンテキストメニュー、イベントの登録
            for(int i = 0; i < layerSize; i++)
            {
                layerStateButtons[i].layerIndex = i;

                // ツールチップ
                layerToolTip.SetToolTip(currentLayerButtons[i], currentLayerButtonsToolTipText);
                layerToolTip.SetToolTip(layerStateButtons[i], layerStateButtonsToolTipText);
                layerToolTip.SetToolTip(layerNameLabels[i], layerNameLabelsToolTipHeadText);
                layerToolTip.SetToolTip(layerNumberLabels[i], layerNameLabelsToolTipHeadText);

                // イベントハンドラの追加
                this.currentLayerButtons[i].MouseUp += new System.Windows.Forms.MouseEventHandler(this.currentLayerRadioButton_MouseUp);
                this.currentLayerButtons[i].KeyUp += new System.Windows.Forms.KeyEventHandler(this.currentLayerRadioButton_KeyUp);
                this.layerStateButtons[i].Click += new System.EventHandler(this.layerStateButton_Click);
                this.layerNameLabels[i].DoubleClick += new System.EventHandler(this.layerNameLabel_DoubleClick);
                this.layerNumberLabels[i].DoubleClick += new System.EventHandler(this.layerNameLabel_DoubleClick);

                // コンテキストメニュー
                this.layerNameLabels[i].ContextMenuStrip = this.layerContextMenuStrip;
                this.layerNumberLabels[i].ContextMenuStrip = this.layerContextMenuStrip;
            }

            // その他のツールチップ
            layerToolTip.SetToolTip(layerGroupNameLabel, layerGroupNameLabelToolTipHeadText);
            layerToolTip.SetToolTip(drawinglayoutSettingButton, drawinglayoutSettingButtonToolTipText);
            layerToolTip.SetToolTip(interlockingDrawingLayoutNameLabel, interlockingDrawingLayoutNameLabelToolTipText);
            layerToolTip.SetToolTip(scaleLabel, scaleLabelToolTipText);
        }

        /// <summary>
        /// コントロールの位置と表示を整える
        /// </summary>
        public void AlignmentAndVisibleControls()
        {
            Debug.Assert(rcAddinControl != null);

            // フォームのレイアウトによる再描画一時停止
            this.SuspendLayout();

            // コンパクトモードフラグ
            bool compactMode = rcAddinControl.IsCompactFormMode();

            // レイヤグループ名、部分図レイアウト情報の表示・非表示
            layerGroupNameLabel.Visible = compactMode ? false : true;
            drawingLyaoutBox.Visible = compactMode ? false : true;
            drawinglayoutSettingButton.Visible = compactMode ? false : true;
            interlockingDrawingLayoutNameLabel.Visible = compactMode ? false : true;
            scaleLabel.Visible = compactMode ? false : true;

            // コントロールのサイズ基準
            Rectangle layerNameLabelDefaultRect = layerNameLabel0.Bounds; // レイヤ名ラベルのRect
            int layerNameLabelDefaultSpace_1_2 = layerNameLabel1.Location.Y - layerNameLabel0.Location.Y - layerNameLabel0.Height;// レイヤ名0とレイヤ名1のラベルの間隔
            
            // layerBox の位置を変えて、その位置を基準にラベルの位置を変えるようにする
            // コンパクト表示モードの場合
            if(compactMode)
                layerBox.Location = layerGroupNameLabel.Location;// レイヤグループ名ラベルの位置にする
            else
                layerBox.Location = new Point(drawingLyaoutBox.Location.X, drawingLyaoutBox.Location.Y + drawingLyaoutBox.Height + (int)(layerNameLabelDefaultRect.Height * 0.4));
            
            // 各レイヤのコントロールの位置とサイズはレイヤ0のコントロールに合わせる(縦方向の間隔はレイヤ0とレイヤ1のレイヤ名ラベルの間隔を基準にする)
            int currentLayerButtonsMarginY = (int)((layerNameLabelDefaultRect.Height - currentLayerRadioButton0.Size.Height) * 0.5);
            int layerStateButtonsMarginY = (int)((layerNameLabelDefaultRect.Height - layerStateButton0.Size.Height) * 0.5);
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                // 位置サイズ調整
                if(i == 0)
                {
                    // レイヤ名ラベル
                    layerNameLabels[0].Visible = compactMode ? false : true;

                    // カレントボタン
                    currentLayerButtons[0].Location = new Point(currentLayerButtons[0].Location.X, layerNameLabels[0].Location.Y + currentLayerButtonsMarginY);
                    // 状態ボタン
                    layerStateButtons[0].Location = new Point(layerStateButtons[0].Location.X, layerNameLabels[0].Location.Y + layerStateButtonsMarginY);

                    // レイヤ番号ラベル
                    layerNumberLabels[0].Visible = compactMode ? true:false;
                    layerNumberLabels[0].Location = new Point(layerNameLabels[0].Location.X, layerStateButtons[0].Location.Y);
                    layerNumberLabels[0].Size = layerStateButtons[0].Size;
                }
                else
                {
                    // レイヤ名ラベル
                    layerNameLabels[i].Location = new Point(layerNameLabels[i-1].Location.X, layerNameLabels[i-1].Location.Y + layerNameLabelDefaultSpace_1_2 + layerNameLabels[i-1].Size.Height);
                    layerNameLabels[i].Size = layerNameLabels[i-1].Size;
                    layerNameLabels[i].Visible = compactMode ? false : true;

                    // カレントボタン
                    currentLayerButtons[i].Location = new Point(currentLayerButtons[i-1].Location.X, layerNameLabels[i].Location.Y + currentLayerButtonsMarginY);
                    currentLayerButtons[i].Size = currentLayerButtons[i-1].Size;

                    // 状態ボタン
                    layerStateButtons[i].Location = new Point(layerStateButtons[i-1].Location.X, layerNameLabels[i].Location.Y + layerStateButtonsMarginY);
                    layerStateButtons[i].Size = layerStateButtons[i-1].Size;

                    // レイヤ番号ラベル
                    layerNumberLabels[i].Visible = compactMode ? true:false;
                    layerNumberLabels[i].Location = new Point(layerNameLabels[i].Location.X, layerStateButtons[i].Location.Y);
                    layerNumberLabels[i].Size = layerStateButtons[i].Size;
                }

            }

            // レイヤボックス枠
            if(compactMode)
            {
                int rightPointX = layerNumberLabels[layerSize-1].Location.X + (int)(layerNumberLabels[layerSize-1].Size.Width * 1.2);
                layerBox.Width = rightPointX;
            }
            else
            {
                layerBox.Width = drawingLyaoutBox.Width;
            }

            // フォーム自体
            this.Width = layerBox.Width + (int)(layerBox.Left * 1.2);
            int bottomSpace = layerBox.Left;
            this.Height = layerBox.Height + layerBox.Top + bottomSpace;

            // フォームのレイアウトによる再描画処理再開
            this.ResumeLayout();
        }


        /// <summary>
        /// ツールチップ表示のときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerToolTip_Popup(object sender, PopupEventArgs e)
        {
            Control control = e.AssociatedControl;
            if(control == null)
                return;

            string newText = "";

            // レイヤグループ名
            if(control.Equals(layerGroupNameLabel))
                newText = layerGroupNameLabelToolTipHeadText + layerGroupNameLabel.Text;
            else if(control.Equals(drawinglayoutSettingButton))
                newText = drawinglayoutSettingButtonToolTipText;
            else if(control.Equals(interlockingDrawingLayoutNameLabel))
                newText = interlockingDrawingLayoutNameLabelToolTipText;
            else if(control.Equals(scaleLabel))
                newText = scaleLabelToolTipText;

            if(newText != "")
                goto SETTOOLTIP;

            // カレントボタン、レイヤの状態ボタン、レイヤ名ラベル
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                // カレントボタン
                if(control.Equals(currentLayerButtons[i]))
                {
                    newText = currentLayerButtonsToolTipText;
                    goto SETTOOLTIP;
                }
                // レイヤの状態ボタン
                else if(control.Equals(layerStateButtons[i]))
                {
                    newText = layerNameLabelsToolTipHeadText + layerNameLabels[i].Text;
                    newText += "\n";
                    newText += layerStateButtonsToolTipText;
                    goto SETTOOLTIP;
                }
                // レイヤ名ラベル
                else if(control.Equals(layerNameLabels[i]))
                {
                    newText = layerNameLabelsToolTipHeadText + layerNameLabels[i].Text;
                    goto SETTOOLTIP;
                }
                // レイヤ番号ラベル
                else if(control.Equals(layerNumberLabels[i]))
                {
                    newText = layerNameLabelsToolTipHeadText + layerNameLabels[i].Text;
                    goto SETTOOLTIP;
                }
            }

            return;

        SETTOOLTIP:

            // ツールチップのサイズ
            Size orgToolTipSize = TextRenderer.MeasureText(newText, this.Font);
            int toolTipMargin = this.Font.Height;
            Size toolTipSize = new Size(orgToolTipSize.Width + toolTipMargin, orgToolTipSize.Height + toolTipMargin);
            e.ToolTipSize = toolTipSize;

            // ツールチップの文字更新
            if(layerToolTip.GetToolTip(e.AssociatedControl) != newText)
                layerToolTip.SetToolTip(e.AssociatedControl, newText);
    
        }

        /// <summary>
        /// ツールチップのオーナードロー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerToolTip_Draw(object sender, DrawToolTipEventArgs e)
        {
            e.DrawBackground();
            e.DrawBorder();

            int toolTipMargin = this.Font.Height;
            int margin = (int)(toolTipMargin * 0.5);
            Rectangle rText = e.Bounds;
            rText.Offset(margin, margin);

            Color textColor = layerToolTip.ForeColor;
            TextFormatFlags textFormatFlags = TextFormatFlags.Left;
            TextRenderer.DrawText(e.Graphics, e.ToolTipText, this.Font, rText, textColor, textFormatFlags);

        }

        /// <summary>
        /// ドキュメントとレイヤグループセット
        /// </summary>
        /// <param name="activeDoc"></param>
        /// <param name="srcLayerGroup"></param>
        /// <param name="deleteingLayerID"></param>
        /// <returns>変更があったか</returns>
        public void SetLayerGroup(Document activeDoc, Layer srcLayerGroup, int deleteingLayerID = -1)
        {
            if(srcLayerGroup != null)
            {
                Debug.Assert(srcLayerGroup.IsGroup && srcLayerGroup.ID != deleteingLayerID);
            }

            if(this.doc == null && activeDoc == null && this.layerGroup == null && srcLayerGroup == null)
                return;

            bool clearFlg = false;

            if(activeDoc == null)
            {
                clearFlg = true;
            }
            else if(this.doc != null)
            {
                if(!this.doc.Equals(activeDoc))
                    clearFlg = true;
            }

            if(srcLayerGroup == null)
            {
                clearFlg = true;
            }
            else if(this.layerGroup != null)
            {
                if(!this.layerGroup.Equals(srcLayerGroup))
                    clearFlg = true;
            }

            if(clearFlg)
            {
                ClearControls();
            }


            this.doc = activeDoc;
            this.layerGroup = srcLayerGroup;

            if(this.doc == null || this.layerGroup == null)
            {
                this.Enabled = false;// このページは使用不可
                return;
            }
                
            // コントロールを有効に
            this.Enabled = true;

            // レイヤグループ名をセット
            layerGroupNameLabel.Text = layerGroup.Name;
 
            int rcAddInMaxChildLayerSize = RootProCADUserControl.GetRcAddInMaxChildLayerSize();

            // カレントレイヤ情報の更新
            int currentLayerID = UpdateCurrentLayerIdInfo();

            // レイヤのボタンの状態更新
            LayerCollection childLayers = layerGroup.ChildLayers;
           int childLayerCount = 0;
            foreach(Layer childLayer in childLayers)
            {
                // レイヤグループか削除されるレイヤの場合
                if(childLayer.ID == deleteingLayerID || childLayer.IsGroup)
                {
                   continue;
                }
                else
                {
                    // レイヤをレイヤボタンにセット
                    layerStateButtons[childLayerCount].Enabled = true;// コントロール有効
                    layerStateButtons[childLayerCount].layer = childLayer;

                    // カレントレイヤになるボタンはON状態に
                    currentLayerButtons[childLayerCount].Enabled = true;// コントロール有効
                    if(currentLayerID == childLayer.ID)
                    {
                        currentLayerButtons[childLayerCount].Checked = true;
                    }
                    else
                    {
                        currentLayerButtons[childLayerCount].Checked = false;
                    }

                    // レイヤ名
                    layerNameLabels[childLayerCount].Enabled = true;// コントロール有効
                    layerNameLabels[childLayerCount].Text = childLayer.Name;

                    // レイヤ番号
                    layerNumberLabels[childLayerCount].Enabled = true;// コントロール有効

                    // レイヤの最大数確認
                    childLayerCount++;
                    if(childLayerCount >= rcAddInMaxChildLayerSize)
                        break;
                }
            }

            // 空のレイヤボタン
            for(int i = childLayerCount; i < rcAddInMaxChildLayerSize; i++)
            {
                layerStateButtons[i].Enabled = false;
                layerStateButtons[i].layer = null;

                currentLayerButtons[i].Enabled = false;
                currentLayerButtons[i].Checked = false;

                layerNameLabels[i].Enabled = false;
                layerNameLabels[i].Text = "";

                // レイヤ番号
                layerNumberLabels[childLayerCount].Enabled = false;// コントロール有効
            }


            // 連動部分図レイアウトについての処理
            Layout layout = GetInterlockingDrawingLayout();
            // 部分図レイアウト名とスケール値をセット
            if(layout != null)
                UpdateLayoutInfo(layout);
            else
                ClearDrawinglayoutInformation();

            return;
        }

        /// <summary>
        /// このレイヤグループのIDを取得
        /// </summary>
        public int GetLayerGroupID()
        {
            if(this.layerGroup != null)
                return this.layerGroup.ID;

            return -1;
        }

        /// <summary>
        /// このレイヤグループを取得
        /// </summary>
        /// <returns></returns>
        public Layer GetLayerGroup()
        {
            return this.layerGroup;
        }

        /// <summary>
        /// このレイヤグループの名前を取得
        /// </summary>
        /// <returns></returns>
        public string GetLayerGroupName()
        {
            if(this.layerGroup != null)
                return this.layerGroup.Name;

            return "";
        }

        /// <summary>
        /// カレントレイヤ情報の更新
        /// </summary>
        /// <returns></returns>
        private int UpdateCurrentLayerIdInfo()
        {
            Debug.Assert(this.layerGroup != null);
            if(this.layerGroup == null)
                return -1;

            // レイヤグループの説明からカレントレイヤID取得
            int currentLayerId = RootProCADUserControl.GetIdFromLayerGroupDescription(this.layerGroup, RootProCADUserControl.ItemName_CurrentLayerID, out int findFirstIndex, out int findTextLength);
            // Idが有効かチェック
            if(!AddInTools.IsValidLayerID(doc, currentLayerId))
                currentLayerId = -1;

            // 現在のレイヤグループ内のレイヤにドキュメントのカレントレイヤがある場合はそのIDを取得
            int documentCurrentLayerIdInThisLayerGroup = -1;
            LayerCollection childLayers = this.layerGroup.ChildLayers;
            foreach(Layer childLayer in childLayers)
            {
                if(childLayer.IsCurrent)
                {
                    documentCurrentLayerIdInThisLayerGroup = childLayer.ID;
                    break;
                }
            }

            // currentLayerId がマイナス(レイヤグループの説明からIDが取得できなかった)場合やドキュメントのカレントレイヤがレイヤグループ内のレイヤ内にある場合
            if(currentLayerId != documentCurrentLayerIdInThisLayerGroup && documentCurrentLayerIdInThisLayerGroup > 0)
            {
                currentLayerId = documentCurrentLayerIdInThisLayerGroup;
            }

            return currentLayerId;
        }

        /// <summary>
        /// 現在のレイヤグループ内のレイヤにドキュメントのカレントレイヤがあるか
        /// </summary>
        /// <returns></returns>
        public bool IsDocumentCurrentLayer()
        {
            if(this.layerGroup == null)
                return false;

            LayerCollection childLayers = this.layerGroup.ChildLayers;
            foreach(Layer childLayer in childLayers)
            {
                if(childLayer.IsCurrent)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// 子レイヤか
        /// </summary>
        /// <param name="layer"></param>
        /// <returns></returns>
        public bool IsChildLayer(Layer layer)
        {
            if(this.layerGroup == null)
                return false;

            LayerCollection childLayers = this.layerGroup.ChildLayers;
            foreach(Layer childLayer in childLayers)
            {
                if(childLayer.ID == layer.ID)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// 子レイヤのIDか
        /// </summary>
        /// <param name="layer"></param>
        /// <returns></returns>
        public bool IsChildLayerID(int layerID)
        {
            if(this.layerGroup == null)
                return false;

            LayerCollection childLayers = this.layerGroup.ChildLayers;
            foreach(Layer childLayer in childLayers)
            {
                if(childLayer.ID == layerID)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// 指定したレイヤIDのレイヤ状態ボタン取得
        /// </summary>
        /// <param name="layerID"></param>
        /// <returns></returns>
        public LayerStateButton GetLayerStateButton(int layerID)
        {
            int buttonSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < buttonSize; i++)
            {
                Debug.Assert(layerStateButtons[i] != null);
                if(layerStateButtons[i].layer != null)
                {
                    if(layerStateButtons[i].layer.ID == layerID)
                        return layerStateButtons[i];
                }
            }

            return null;
        }

        /// <summary>
        /// 使われていないレイヤの状態ボタンを再描画
        /// </summary>
        public void InvalidateAllEmptyLayerButton()
        {
            int buttonSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < buttonSize; i++)
            {
                Debug.Assert(layerStateButtons[i] != null);
                if(layerStateButtons[i].layer != null)
                {
                    if(!layerStateButtons[i].layer.IsInUse)
                        layerStateButtons[i].Invalidate();
                }
            }
        }

        /// <summary>
        /// 連携している部分図レイアウトIDの取得
        /// </summary>
        /// <returns></returns>
        private int GetInterlockingDrawingLayoutID()
        {
            Layout layout = GetInterlockingDrawingLayout();
            if(layout != null)
                return layout.ID;
 
            return -1;
        }

        /// <summary>
        /// 連携している部分図レイアウト取得
        /// </summary>
        /// <returns></returns>
        public Layout GetInterlockingDrawingLayout()
        {
            Layout layout = null;

            if(this.layerGroup == null)
                return null;

            // レイヤグループの説明からカレント部分図レイアウトID取得
            int interlockingDrawingLayoutID = RootProCADUserControl.GetIdFromLayerGroupDescription(this.layerGroup, RootProCADUserControl.ItemName_InterlockingDrawingLayoutID, out int findFirstIndex, out int findTextLength);
            if(interlockingDrawingLayoutID >= 0)
                layout = AddInTools.GetLayoutOfID(this.doc, interlockingDrawingLayoutID);

            return layout;
        }

       
        /// <summary>
        /// 各コントロールの内容をクリア
        /// </summary>
        /// <param name="enabled"></param>
        public void ClearControls(bool enabled = false)
        {
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                // カレントレイヤボタンOff
                currentLayerButtons[i].Checked = false;
                currentLayerButtons[i].Enabled = enabled;

                // ボタンのレイヤ
                layerStateButtons[i].layer = null;
                layerStateButtons[i].Enabled = enabled;

                // レイヤ名
                layerNameLabels[i].Text = "";
                layerNameLabels[i].Enabled = enabled;
            }

            // レイヤグループ名
            layerGroupNameLabel.Text = "";

            // 部分図レイアウト情報表示のクリア
            ClearDrawinglayoutInformation();

            // null セット
            this.doc = null;
            this.layerGroup = null;
        }

        /// <summary>
        /// 部分図レイアウト情報表示のクリア
        /// </summary>
        public void ClearDrawinglayoutInformation()
        {
            // 部分図レイアウト名
            interlockingDrawingLayoutNameLabel.Text = "";

            // スケール値ラベルと
            scaleLabel.Text = "";
        }

        /// <summary>
        /// 連携部分図レイアウトの名前取得
        /// </summary>
        public string GetInterlockingDrawingLayoutName()
        {
            return interlockingDrawingLayoutNameLabel.Text;
        }

        /// <summary>
        /// 指定したレイヤグループまたはレイヤの状態コントロール再描画
        /// </summary>
        /// <param name="layer"></param>
        public void UpdateLayerState(Layer layer)
        {
            Debug.Assert(layer != null);
            if(layer == null)
                return;

            bool isGroup = false;
            if(layer.IsGroup)
            {
                Debug.Assert(layer.ID == layerGroup.ID);
                isGroup = true;
            }

            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                if(layerStateButtons[i].layer != null)
                {
                    if(isGroup || layerStateButtons[i].layer.ID == layer.ID)
                    {
                        currentLayerButtons[i].Invalidate();
                        layerStateButtons[i].Invalidate();

                        if(isGroup)
                            layerGroupNameLabel.Text = layer.Name;
                        else
                            layerNameLabels[i].Text = layer.Name;

                        if(!isGroup)
                            break;
                    }
                }
            }

            // 説明の中から関連づいている部分図レイアウト情報更新
            if(isGroup)
                UpdateLayoutInfo(layer);
        }


        public void InvalidateLayerStateButtons()
        {
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                if(layerStateButtons[i].layer != null)
                {
                    layerStateButtons[i].Invalidate();
                }
            }
        }

        /// <summary>
        /// 部分図レイアウト状態表示更新
        /// </summary>
        /// <param name="layerGroup"></param>
        public void UpdateLayoutInfo(Layer layerGroup)
        {
            if(layerGroup == null)
            {
                ClearDrawinglayoutInformation();
                return;
            }

            Debug.Assert(layerGroup.IsGroup);
            if(!layerGroup.IsGroup)
                return;

            // レイヤグループの説明文字列から指定した項目名のIDを取得
            int currentLayoutID = RootProCADUserControl.GetIdFromLayerGroupDescription(layerGroup, RootProCADUserControl.ItemName_InterlockingDrawingLayoutID, out int dummyIndex1, out int dummyIndex2);

            Layout layout = AddInTools.GetLayoutOfID(this.doc, currentLayoutID);
            UpdateLayoutInfo(layout);
        }

        /// <summary>
        /// 部分図レイアウト状態表示更新
        /// </summary>
        /// <param name="layout"></param>
        public void UpdateLayoutInfo(Layout layout)
        {
            if(layout != null)
            {
                interlockingDrawingLayoutNameLabel.Text = layout.Name;

                // スケール
                double scaleXNumerator = layout.ScaleXNumerator;      // 分子値
                double scaleXDenominator = layout.ScaleXDenominator;  // 分母値
                string scaleText = Properties.Resources.scale;// スケール
                scaleText += ": " + scaleXNumerator.ToString() + " / " + scaleXDenominator.ToString();
                scaleLabel.Text = scaleText;
            }
            else
            {
                ClearDrawinglayoutInformation();
            }
        }

        /// <summary>
        /// レイヤの状態をトルグして切り替える
        /// </summary>
        /// <param name="layerStateButton"></param>
        private void TorugLayerState(LayerStateButton layerStateButton)
        {
            if(layerStateButton.layer == null)
                return;

            Layer layer = layerStateButton.layer;
            ShapeState shapeState = layer.State;
            // enum ShapeState
            // Editable = 0,// 編集可能
            // Visible = 1, // 表示のみ
            // NotVisible = 2 // 表示しない

            // カレントの場合は何もしない
            if(layer.IsCurrent)
                return;

            // UndoUnitの開始
            doc.UndoManager.BeginUndoUnit();

            // 切り替える
            if(shapeState == ShapeState.Editable)
            {
                layer.State = ShapeState.Visible;
            }
            else if(shapeState == ShapeState.Visible)
            {
                layer.State = ShapeState.NotVisible;
            }
            else
            {
                layer.State = ShapeState.Editable;
            }

            // UndoUnitの終了
            doc.UndoManager.EndUndoUnit();

        }

        /// <summary>
        /// カレントレイヤセット
        /// </summary>
        /// <param name="layerStateButtonIndex"></param>
        private void SetCurrentLayer(int layerStateButtonIndex)
        {
            if(doc == null)
                return;

            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            Debug.Assert(layerStateButtonIndex < layerSize);

            // レイヤをカレントにする
            Layer layer = layerStateButtons[layerStateButtonIndex].layer;
            if(layer != null)
            {
                // UndoUnitの開始
                bool beginUndoUnitFlg = false;
                if(!doc.UndoManager.IsInUndoUnit)
                {
                    doc.UndoManager.BeginUndoUnit();
                    beginUndoUnitFlg = true;
                }

                doc.LayerTable.CurrentLayer = layer;

                // UndoUnitの終了
                if(beginUndoUnitFlg)
                {
                    doc.UndoManager.EndUndoUnit();
                }
            }
        }

        /// <summary>
        /// カレントレイヤセット
        /// </summary>
        /// <param name="radioButton"></param>
         private void SetCurrentLayer(RadioButton radioButton)
        {
            int index = -1;
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                if(currentLayerButtons[i].Equals(radioButton))
                    index = i;
            }

            if(index < 0)
                return;

            SetCurrentLayer(index);
        }

        private void SetCurrentLayer(Label label)
        {
            int index = -1;
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                if(layerNameLabels[i].Equals(label) || layerNumberLabels[i].Equals(label))
                    index = i;
            }

            if(index < 0)
                return;

            SetCurrentLayer(index);
        }


        /// <summary>
        /// カレントレイヤラジオボタンのマウスボタンアップイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void currentLayerRadioButton_MouseUp(object sender, MouseEventArgs e)
        {
            // 左ボタンの場合
            if(e.Button == MouseButtons.Left)
            {
                // カレントレイヤ切り替え
                RadioButton radioButton = (RadioButton)sender;
                SetCurrentLayer(radioButton);
            }
        }

        private void currentLayerRadioButton_KeyUp(object sender, KeyEventArgs e)
        {
            // スペースキーの場合
            if(e.KeyData == Keys.Space)
            {
                // カレントレイヤ切り替え
                RadioButton radioButton = (RadioButton)sender;
                SetCurrentLayer(radioButton);
            }
        }

        /// <summary>
        /// レイヤ名のラベルをダブルクリックしたときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerNameLabel_DoubleClick(object sender, EventArgs e)
        {
            // カレントレイヤ切り替え
            SetCurrentLayer((Label)sender);
        }

        public int GetCheckedCurrentLayerRadioButtonLayerID()
        {
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            for(int i = 0; i < layerSize; i++)
            {
                // カレントレイヤボタン
                if(currentLayerButtons[i].Checked == true)
                {
                    Layer layer = layerStateButtons[i].layer;
                    if(layer != null)
                        return layer.ID;
                }
            }

            return -1;
        }

        /// <summary>
        /// レイヤ状態ボタンクリックイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerStateButton_Click(object sender, EventArgs e)
        {
            // レイヤの状態変更
            LayerStateButton layerStateButton = (LayerStateButton)sender;
            TorugLayerState(layerStateButton);
        }

        /// <summary>
        /// レイヤの状態や名前を表すコントロールのインデックスを取得
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        private int GetLayerStateButtonIndex(Control source)
        {
            int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
            int layerStateButtonIndex = -1;

            // レイヤ状態ボタンの場合
            if(source.GetType().Equals(typeof(LayerStateButton)))
            {
                for(int i = 0; i < layerSize; i++)
                {
                    if(layerStateButtons[i].Equals(source))
                        layerStateButtonIndex = i;
                }
            }
            // カレントレイヤにするラジオボタンの場合
            else if(source.GetType().Equals(typeof(RadioButton)))
            {
                for(int i = 0; i < layerSize; i++)
                {
                    if(currentLayerButtons[i].Equals(source))
                        layerStateButtonIndex = i;
                }
            }
            // レイヤ名かレイヤ番号のラベルの場合
            else if(source.GetType().Equals(typeof(Label)))
            {
                for(int i = 0; i < layerSize; i++)
                {
                    if(layerNameLabels[i].Equals(source) || layerNumberLabels[i].Equals(source))
                        layerStateButtonIndex = i;
                }
            }

            return layerStateButtonIndex;
        }
        
        /// <summary>
        /// コンテキストメニューオープン イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            lastContextMenuSelectLayerIndex = -1;

            ContextMenuStrip menu = (ContextMenuStrip)sender;
            Control source = menu.SourceControl;

            int layerStateButtonIndex = GetLayerStateButtonIndex(source);
            if(layerStateButtonIndex >= 0)
            {
                LayerStateButton layerStateButton = layerStateButtons[layerStateButtonIndex];

                Layer layer = layerStateButton.layer;
                ShapeState shapeState = layer.State;

                // 編集可能
                editLayerToolStripMenuItem.Checked = shapeState == ShapeState.Editable;

                // 表示のみ
                lockLayerToolStripMenuItem.Checked = shapeState == ShapeState.Visible;
                
                // 表示しない
                undisplayLayerToolStripMenuItem.Checked = shapeState == ShapeState.NotVisible;

                // カレントレイヤ
                currentLayerToolStripMenuItem.Checked = layer.IsCurrent;

                // 隠しレイヤ
                hideLayerToolStripMenuItem.Checked = layer.Hide;

                // 印刷する
                printLayerToolStripMenuItem.Checked = layer.Printable;

                // 削除
                deleteToolStripMenuItem.Enabled = layer.IsCurrent ? false : true;

                // 選んだレイヤのインデックス更新
                lastContextMenuSelectLayerIndex = layerStateButtonIndex;
            }

        }

        /// <summary>
        /// コンテキストメニューから「編集可能」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void editLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                layerStateButtons[lastContextMenuSelectLayerIndex].layer.State = ShapeState.Editable;

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }
        
        /// <summary>
        /// コンテキストメニューから「表示のみ」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lockLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                layerStateButtons[lastContextMenuSelectLayerIndex].layer.State = ShapeState.Visible;

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }

        /// <summary>
        /// コンテキストメニューから「表示しない」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void undisplayLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                layerStateButtons[lastContextMenuSelectLayerIndex].layer.State = ShapeState.NotVisible;

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }

        /// <summary>
        /// コンテキストメニューから「カレントレイヤ」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void currentLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                SetCurrentLayer(lastContextMenuSelectLayerIndex);
            }
        }

        /// <summary>
        /// コンテキストメニューから「隠しレイヤ」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void hideLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                layerStateButtons[lastContextMenuSelectLayerIndex].layer.Hide = !layerStateButtons[lastContextMenuSelectLayerIndex].layer.Hide;

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }

        /// <summary>
        /// コンテキストメニューから「印刷する」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void printLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);
                
                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();
                
                layerStateButtons[lastContextMenuSelectLayerIndex].layer.Printable = !layerStateButtons[lastContextMenuSelectLayerIndex].layer.Printable;

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤ」について変更が選ばれた時の処理
        /// </summary>
        /// <param name="state">表示状態：編集可能、表示のみ、表示しない</param>
        /// <param name="hideFlgValue">-1:表示状態を優先設定をする, 0:隠しレイヤOFF, 1:隠しレイヤ</param>
        private void OtherLayerToolStripMenuItemClick(ShapeState state = ShapeState.Editable, int hideFlgValue = -1)
        {
            // このレイヤのID
            int notChangeLayerID = -1;

            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);
                notChangeLayerID = layerStateButtons[lastContextMenuSelectLayerIndex].layer.ID;
            }

            if(notChangeLayerID < 0)
                return;

            if(doc == null)
                return;            

            // 子レイヤのレイヤコレクション取得
            LayerTable layerTable = doc.LayerTable;
            LayerCollection layerCollection = layerTable.RootLayer.ChildLayers;

            // UndoUnitの開始
            doc.UndoManager.BeginUndoUnit();

            // 他のレイヤ状態を変更
            RootProCADUserControl.SetLayerGroupState(layerCollection, notChangeLayerID, false, state, hideFlgValue);

            //UndoUnitの終了
            doc.UndoManager.EndUndoUnit();
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤを」-「編集可能」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otherEditLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OtherLayerToolStripMenuItemClick(ShapeState.Editable);
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤを」-「表示のみ」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otherLockLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OtherLayerToolStripMenuItemClick(ShapeState.Visible);
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤを」-「表示しない」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otherUndisplayLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OtherLayerToolStripMenuItemClick(ShapeState.NotVisible);
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤを」-「隠しレイヤ」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otherHideLayerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OtherLayerToolStripMenuItemClick(0, 1);
        }

        /// <summary>
        /// コンテキストメニューから「他のレイヤを」-「隠しレイヤ OFF」が選ばれた場合
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void otherHideLayerOffToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OtherLayerToolStripMenuItemClick(0, 0);
        }

        /// <summary>
        /// コンテキストメニューからレイヤの「削除」を選んだとき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);
                Layer layer = layerStateButtons[lastContextMenuSelectLayerIndex].layer;
                Debug.Assert(!layer.IsGroup);

                // カレントでないこと
                Debug.Assert(!layer.IsCurrent);

                // 削除していいレイヤか
                if(!AddInTools.IsDeleteableLayer(doc, layer))
                    return;

                // 図形に使われているレイヤだった場合削除していいか問い合わせ
                if(!AddInTools.CheckAndQuestionDeleteUseLayerShape(doc, layer))
                    return;

                Debug.Assert(layerGroup != null);
                LayerCollection childLayers = layerGroup.ChildLayers;

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                // 削除
                childLayers.RemoveByID(layer.ID);

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }
        }

        /// <summary>
        /// コンテキストメニューから「名前を変更」を選んだとき
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void editlyaerNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
             if(doc == null)
                return;       

            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);
                Layer layer = layerStateButtons[lastContextMenuSelectLayerIndex].layer;

                EditLayerNameForm dlg = new EditLayerNameForm(doc, layer);
                if(dlg.ShowDialog() == DialogResult.OK)
                {
                    //レイヤ名（事前に他のレイヤで同じ名前がないかチェックすること）
                    string layerName = dlg.GetName();
                    if(layer.Name != layerName)
                    {
                        // UndoUnitの開始
                        doc.UndoManager.BeginUndoUnit();

                        // 名称変更
                        layer.Name = layerName;

                        // UndoUnitの終了
                        doc.UndoManager.EndUndoUnit();
                    }
                }
            }
        }

        /// <summary>
        /// スケール連携設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawinglayoutSettingButton_Click(object sender, EventArgs e)
        {
            // 部分図レイアウトのスケール連携設定
            InterlockingDrawinglayoutSetting();
        }

        /// <summary>
        /// 部分図レイアウトのスケール連携設定
        /// </summary>
        public void InterlockingDrawinglayoutSetting()
        {
            if(this.layerGroup == null)
                return;

            // レイヤグループの説明からカレント部分図レイアウトID取得
            int selectLayoutID = RootProCADUserControl.GetIdFromLayerGroupDescription(this.layerGroup, RootProCADUserControl.ItemName_InterlockingDrawingLayoutID, out int findFirstIndex, out int findTextLength);

            DrawinglayoutSettingForm dlg = new DrawinglayoutSettingForm(this);
            dlg.layerGroupNameLabel.Text = this.layerGroup.Name;
            dlg.SetDrawingLayouts(doc.Drawings, selectLayoutID);// 部分図レイアウト情報のセット
            if(dlg.ShowDialog() == DialogResult.OK)
            {
                double scaleNumerator = 1.0;
                double scaleDenominator = 1.0;

                // 選ばれた部分図レイアウトのIDをレイヤグループの説明に書き込む(selectedLayoutID < 0 の場合は削除してリンクを切る)
                int selectedLayoutID = -1;
                Layout selectedLayout = dlg.GetSelectedLayout();
                if(selectedLayout != null)
                {
                    selectedLayoutID = selectedLayout.ID;

                    // 部分図レイアウトの分子
                    bool result1 = dlg.GetScaleNumerator(out scaleNumerator);
                    if(result1 == false || scaleNumerator < Precision.Confusion)
                    {
                        Debug.Assert(false);
                        return;
                    }
                    // 部分図レイアウトの分母
                    bool result2 = dlg.GetScaleDenominator(out scaleDenominator);
                    if(result2 == false || scaleDenominator < Precision.Confusion)
                    {
                        Debug.Assert(false);
                        return;
                    }
                }

                // UndoUnitの開始
                doc.UndoManager.BeginUndoUnit();

                // 書込み
                RootProCADUserControl.SetIdToLayerGroupDescription(this.layerGroup, RootProCADUserControl.ItemName_InterlockingDrawingLayoutID, selectedLayoutID);

                if(selectedLayoutID >= 0)
                {
                    // カレント部分図レイアウトも切り替える
                    // このレイヤグループ内のレイヤにドキュメントのカレントレイヤがなければ有効なレイヤの先頭のをカレントにする
                    if (!IsDocumentCurrentLayer())
                    {
                        int layerSize = RootProCADUserControl.RcAddInChildLayerSize;
                        for (int i = 0; i < layerSize; i++)
                        {
                            if (layerStateButtons[i].layer != null)
                            {
                                SetCurrentLayer(i);
                                break;
                            }
                        }
                    }

                    // XY 異スケールにはしない
                    selectedLayout.EnableScaleY = false;
                    // スケール値セット
                    selectedLayout.ScaleXNumerator = scaleNumerator;
                    selectedLayout.ScaleXDenominator = scaleDenominator;

                    if(IsDocumentCurrentLayer())
                        doc.CurrentLayout = selectedLayout;
                }
                else
                {
                    // 部分図レイアウト名とスケールは消しておく
                    ClearDrawinglayoutInformation();

                    // 「なし」にした場合、現在のカレント部分図レイアウトは用紙にする
                    Layout layout = AddInTools.GetPaperDrawingLayout(doc);
                    if(layout != null)
                    {
                        doc.CurrentLayout = layout;

                        // 警告メッセージ

                        string msg = interlockingDrawinglayoutSetting_InformationMessage1;
                        MessageBox.Show(msg, Properties.Resources.addInProgramName, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }

                // UndoUnitの終了
                doc.UndoManager.EndUndoUnit();
            }

            // 再描画
            UpdateLayerState(this.layerGroup);
        }

        /// <summary>
        /// 指定したIDの部分図レイアウトが他のレイアグループで連動していないか
        /// </summary>
        /// <param name="lLayoutID"></param>
        /// <returns></returns>
        public bool IsInterlockingDrawinglayoutAtOtherLayerGroup(int layoutID)
        {
            Debug.Assert(rcAddinControl != null);
            Debug.Assert(layerGroup != null);

            bool stat = rcAddinControl.IsInterlockingDrawinglayoutAtOtherLayerGroup(layoutID, layerGroup.ID);

            return stat;
        }

        /// <summary>
        /// レイヤグループのレイヤは使われているか
        /// </summary>
        /// <returns></returns>
        public bool IsInUseLayerGroup()
        {
            if(this.layerGroup == null)
                return false;

            bool isUse = this.layerGroup.IsInUse;

            return isUse;
        }

        /// <summary>
        /// レイヤのプロパティ表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void layerPropertyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // コンテキストメニューオープン時のコントロールのインデックスからどのレイヤかを決める
            if(lastContextMenuSelectLayerIndex >= 0 && lastContextMenuSelectLayerIndex < layerStateButtons.Count())
            {
                Debug.Assert(layerStateButtons[lastContextMenuSelectLayerIndex].layer != null);
                Debug.Assert(rcAddinControl != null);

                // プロパティウィンドウにレイヤのプロパティを表示させる
                Layer layer = layerStateButtons[lastContextMenuSelectLayerIndex].layer;
                rcAddinControl.ShowLayerProperty(layer);
            }
        }
    }
}
