﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;

using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

namespace RCAddInSelectLayerGroupCS
{
    public partial class LayerStateButton:Button
    {
        public int layerIndex = -1;
        public Layer layer = null;

        Image layerStateCurrentImage = null;
        Image layerStateEditImage = null;
        Image layerStateUndisplayImage = null;
        Image layerStateCurrentPenImage = null;
        Image layerStateCurrentPen_HideImage = null;
        Image layerStateLockImage = null;
        Image layerStateLock_HideImage = null;

        public LayerStateButton()
        {
            InitializeComponent();

            this.DoubleBuffered = true;

            layerStateCurrentImage = Properties.Resources.LayerStateCurrent;
            layerStateEditImage = Properties.Resources.LayerStateEdit;
            layerStateUndisplayImage = Properties.Resources.LayerStateUndisplay;
            layerStateCurrentPenImage = Properties.Resources.LayerStateCurrentPen;
            layerStateCurrentPen_HideImage = Properties.Resources.LayerStateCurrentPen_Hide;
            layerStateLockImage = Properties.Resources.LayerStateLock;
            layerStateLock_HideImage = Properties.Resources.LayerStateLock_Hide;
        }

        private void layerButton_Paint(object sender, PaintEventArgs e)
        {
             Button btn = (Button)sender;

            // SmoothingMode
			e.Graphics.SmoothingMode = SmoothingMode.HighQuality;

            // テキストを取得
            //string txt = btn.Text;// layerIndex.ToString("X");// 16進数大文字に;

            // レイヤの状態
            ShapeState shapeState = (ShapeState)(-1);// 未定義
            if(layer != null)
                shapeState = layer.State;
            // enum ShapeState
            // Editable = 0,// 編集可能
            // Visible = 1, // 表示のみ
            // NotVisible = 2 // 表示しない

            // カレントレイヤか
            bool isCurrentLayer = false;
            if(layer != null)
                isCurrentLayer = layer.IsCurrent;

            // 隠しレイヤか
            bool isHide = false;
            if(layer != null)
                isHide = layer.Hide;

            // イメージ範囲
            Rectangle imageRectangle = btn.ClientRectangle;
            imageRectangle.Inflate(-4, -4);
             
            // カレントレイヤの場合
            if (isCurrentLayer)
            {
                e.Graphics.DrawImage(layerStateCurrentImage, imageRectangle);
            }
            // 隠しレイヤの場合
            else if(isHide)
            {
                btn.ForeColor = btn.BackColor;// ボタンの文字の色もバックカラーと同じにする（イメージが透過で下の文字が見えないように背景色と同じにしているが文字はニーモニックキーは効く）
                e.Graphics.DrawImage(layerStateUndisplayImage, imageRectangle);
            }
            // 編集可能
            else if(shapeState == ShapeState.Editable)
            {
                e.Graphics.DrawImage(layerStateEditImage, imageRectangle);
            }
            // 表示のみ
            else if(shapeState == ShapeState.Visible)
            {
                e.Graphics.DrawImage(layerStateEditImage, imageRectangle);
            }
            // 表示しない
            else if(shapeState == ShapeState.NotVisible)
            {
                btn.ForeColor = btn.BackColor;// ボタンの文字の色もバックカラーと同じにする（イメージが透過で下の文字が見えないように背景色と同じにしているが文字はニーモニックキーは効く）
                e.Graphics.DrawImage(layerStateUndisplayImage, imageRectangle);
            }
            else
            {
                // 文字の周りの円
                Pen pen = new Pen(Color.Gray, 1);
                int drawSize = (int)(btn.ClientRectangle.Height * 0.6);
                int topRightX = btn.ClientRectangle.X + (int)((btn.ClientRectangle.Width - drawSize) * 0.5);
                int topRightY = btn.ClientRectangle.Y + (int)((btn.ClientRectangle.Height - drawSize) * 0.5);
                Rectangle rect = new Rectangle(topRightX, topRightY, drawSize, drawSize);
                e.Graphics.DrawArc(pen, rect, 0, 360);
            }

            // 鍵マーク
            if(shapeState == ShapeState.Visible)
            {
                Rectangle imageRectangle2 = imageRectangle;
                imageRectangle2.Inflate(-3, -3);
                e.Graphics.DrawImage(layerStateLockImage, imageRectangle2);
            }

            // カレントの場合上にペンのイメージを載せる
            if(isCurrentLayer)
                e.Graphics.DrawImage(layerStateCurrentPenImage, imageRectangle);

            // 使われているレイヤの場合のマーク
            if(layer != null && layer.IsInUse)
            {
                Pen linePen = new Pen(Color.Orange, 3);
                Point p1 = new Point(imageRectangle.X, imageRectangle.Y);
                Point p2 = new Point(imageRectangle.X + imageRectangle.Width / 5, p1.Y);
                e.Graphics.DrawLine(linePen, p1, p2);
            }
        }

    }
}
