﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using RootPro.RootProCAD;

namespace RCAddInSelectLayerGroupCS
{
    /// <summary>
    /// このアドインで使う TabControl
    /// </summary>
    public partial class RcAddInSelectLayerGroupTabControl : TabControl
    {
        // マウスがタブコントロールの領域内に入ったときのタブのインデックス
        public int onMouseTabIndex = -1;

        // タブに付けるイメージ
        Image layerGroupCloseImage = null;
        Image layerGroupOpenImage = null;
        Image layoutCurrentImage = null;
        Image layoutEditImage = null;
        Image layoutUndisplayImage = null;
        Image layoutLockImage = null;
        Image layoutCurrentPenImage = null;

        public RcAddInSelectLayerGroupTabControl()
        {
            InitializeComponent();

            // Paintイベントで描画できるようにする
            this.SetStyle(ControlStyles.UserPaint, true);

            // ダブルバッファリングを有効にする
            this.DoubleBuffered = true;

            // リサイズで再描画する
            this.ResizeRedraw = true;

            // タブは左
            this.Alignment = TabAlignment.Left;

            this.Appearance = TabAppearance.Normal;

            // タブに付けるイメージ
            layerGroupCloseImage = Properties.Resources.LayerGroupClose;
            layerGroupOpenImage = Properties.Resources.LayerGroupOpen;
            layoutCurrentImage = Properties.Resources.LayoutCurrent;
            layoutEditImage = Properties.Resources.LayoutEdit;
            layoutUndisplayImage = Properties.Resources.LayoutUndisplay;
            layoutLockImage = Properties.Resources.LayerStateLock;
            layoutCurrentPenImage = Properties.Resources.LayerStateCurrentPen;
        }

        /// <summary>
        /// OnPaint
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e)
        {
            // SmoothingMode
			e.Graphics.SmoothingMode = SmoothingMode.HighQuality;

            base.OnPaint(e);

            // TabControlの背景を塗る
            e.Graphics.FillRectangle(SystemBrushes.Control, this.ClientRectangle);

            if(this.TabPages.Count == 0)
                return;

            Debug.Assert(this.Alignment == TabAlignment.Left);

            // TabPage の枠を描画
            TabPage page = this.TabPages[this.SelectedIndex];
            Rectangle pageRect = new Rectangle(page.Bounds.X - 2, page.Bounds.Y - 2, page.Bounds.Width + 5, page.Bounds.Height + 5);
            TabRenderer.DrawTabPage(e.Graphics, pageRect);

            // タブを描画
            for(int i = 0; i < this.TabPages.Count; i++)
            {
                page = this.TabPages[i];
                Rectangle tabRect = this.GetTabRect(i);

                // 表示するタブの状態を決定する
                System.Windows.Forms.VisualStyles.TabItemState state;
                if (!this.Enabled)
                    state = System.Windows.Forms.VisualStyles.TabItemState.Disabled;
                else if(this.SelectedIndex == i)
                    state = System.Windows.Forms.VisualStyles.TabItemState.Selected;
                else
                    state = System.Windows.Forms.VisualStyles.TabItemState.Normal;

                // 選択されたタブとページの間の境界線を消すために描画する範囲を大きくする
                if(this.SelectedIndex == i)
                     tabRect.Width += 1;
 
                // 画像のサイズを決定する
                Size imgSize = new Size(tabRect.Height, tabRect.Width);;

                // Textは表示させない
                string tabText = "";

                // タブの画像を作成する
                Bitmap bmp = new Bitmap(imgSize.Width, imgSize.Height);
                Graphics g = Graphics.FromImage(bmp);
                TabRenderer.DrawTabItem(g, new Rectangle(0, 0, bmp.Width, bmp.Height + 1), tabText, page.Font, false, state);
                g.Dispose();
                // 画像を回転
                bmp.RotateFlip(RotateFlipType.Rotate270FlipNone);
                // 画像を描画
                e.Graphics.DrawImage(bmp, tabRect.X, tabRect.Y, bmp.Width, bmp.Height);

                // レイヤグループのタブコントロール描画処理
                layerGroupTabControl_DrawItem(i, e);

                bmp.Dispose();
            }

        }

        /// <summary>
        /// レイヤグループのタブコントロール描画処理
        /// </summary>
        /// <param name="tabIndex"></param>
        /// <param name="e"></param>
        private void layerGroupTabControl_DrawItem(int tabIndex, PaintEventArgs e)
        {
            Debug.Assert(tabIndex >= 0 && tabIndex < this.TabPages.Count);

            // SmoothingMode
			e.Graphics.SmoothingMode = SmoothingMode.HighQuality;

            // タブページの LayerGroupsLayerControl 取得
            LayerGroupsLayerControl layerGroupsLayerControl = GetLayerGroupsLayerControl(tabIndex);
            if(layerGroupsLayerControl == null)
                return;

            TabPage tabPage = this.TabPages[tabIndex];

            // タブページのテキストを取得
            string text = tabPage.Text;

            // タブの範囲
            Rectangle tabRect = this.GetTabRect(tabIndex);
            
            // タブに付けるイメージの範囲
            // グループイメージ
            Image iconImage = null;
            int drawImageSize = (int)(tabRect.Height * 0.6);
            int imageTopLeftX = tabRect.X + 6;
            int imageTopLeftY = tabRect.Y + (int)((tabRect.Height - drawImageSize) * 0.5);
            Rectangle imageDrawRect = new Rectangle(imageTopLeftX, imageTopLeftY, drawImageSize, drawImageSize);
            
            // 部分図レイアウトの状態イメージ
            Image iconImage2 = null;
            Rectangle imageDrawRect2 = new Rectangle(imageTopLeftX + drawImageSize + 3, imageTopLeftY, drawImageSize, drawImageSize);

            // 部分図レイアウトのカレントor表示のみの場合に重ねるイメージ
            Image iconImage3 = null;
            Rectangle imageDrawRect3 = imageDrawRect2;
            imageDrawRect3.Inflate(-3, -3);

            // タブのテキストと背景を描画するためのブラシとペン
            SolidBrush selectTabBackBrush = null;
            Color textColor = Color.Empty;// 文字の色
            Pen pen = null; // 枠の

            // 実際のドキュメントのカレントレイヤと同じレイヤがあるタブは赤い四角で囲む
            bool isDocumentCurrentLayerTab = false;
            if(layerGroupsLayerControl.IsDocumentCurrentLayer())
                 isDocumentCurrentLayerTab = true;

            // 文字の色(選択されているタブ)
            if(tabIndex == this.SelectedIndex && layerGroupsLayerControl.Enabled)
            {
                // タブページの文字と同じ色にする
                textColor = tabPage.ForeColor;
            }
            else
            {
                if(layerGroupsLayerControl.Enabled)
                    textColor = Color.Black;
                else
                    textColor = Color.Gray;
            }

            // レイヤグループ番号を囲むペンの色
            if(isDocumentCurrentLayerTab)
            {
                pen = new Pen(Color.Red, 2);
            }
            else
            {
                if(layerGroupsLayerControl.Enabled)
                    pen = new Pen(Color.Black, 1);
                else
                    pen = new Pen(Color.Gray, 1);
            }

            // 選択されているタブ
            if(tabIndex == this.SelectedIndex)
            {
                if(layerGroupsLayerControl.Enabled)
                {
                    // 背景はタブページと同じ色にする
                    Color selectTabBackColor = tabPage.BackColor;
                    selectTabBackBrush = new SolidBrush(selectTabBackColor);
                }

                iconImage = layerGroupOpenImage;// レイヤーグループオープンイメージ
            }
            // 選択されていないタブ
            else
            {
                iconImage = layerGroupCloseImage;// レイヤーグループクローズイメージ
            }

            // レイヤグループは使われているかフラグ
            bool isInUseLayerGroup = layerGroupsLayerControl.IsInUseLayerGroup();
            // 部分図に図形はあるかフラグ
            bool isShapesInDrawing = false;

            // 部分図レイアウトのアイコン
            string interlockingDrawingLayoutName = layerGroupsLayerControl.GetInterlockingDrawingLayoutName();
            Layout layout = null;
            if(interlockingDrawingLayoutName != "")
            {
                layout = layerGroupsLayerControl.GetInterlockingDrawingLayout();
                if(layout != null)
                {
                    // このレイアウトの部分図に図形があるか
                    isShapesInDrawing = layout.Drawing.Shapes.Count > 0 ? true : false;

                    // カレント部分図レイアウト(部分図自体がカレントの場合も)
                    if(layout.IsCurrent || layout.Drawing.IsCurrent)
                    {
                        iconImage2 = layoutCurrentImage;
                        iconImage3 = layoutCurrentPenImage;
                    }
                    // 編集
                    else if(layout.Drawing.Editable)
                    {
                        iconImage2 = layoutEditImage;
                    }
                    // 表示のみ
                    else if(layout.Drawing.State == ShapeState.Visible)
                    {
                        iconImage2 = layoutEditImage;
                        iconImage3 = layoutLockImage;
                    }
                    // 表示しない
                    else
                    {
                        iconImage2 = layoutUndisplayImage;
                    }
                }
            }

            // 背景の描画
            if(selectTabBackBrush != null)
            {
                Rectangle fillRect = tabRect;
                Rectangle tabRect2 = this.GetTabRect(tabIndex);

                int offsetX = 2;
                fillRect.Y = tabRect2.Y + offsetX;
                fillRect.Height = tabRect2.Height - (int)(offsetX * 1.5);
                fillRect.X = tabRect2.X + 1;

                // 選択されたタブとページの間の境界線を消すために描画する範囲を大きくする
                if(tabIndex == this.SelectedIndex)
                     fillRect.Width += 1;

                e.Graphics.FillRectangle(selectTabBackBrush, fillRect);
            }

            // グループアイコンイメージ
            if(iconImage != null && layerGroupsLayerControl.Enabled)
                e.Graphics.DrawImage(iconImage, imageDrawRect);

            // 部分図レイアウトアイコンイメージ
            if(iconImage2 != null && layerGroupsLayerControl.Enabled)
                e.Graphics.DrawImage(iconImage2, imageDrawRect2);

            // 部分図レイアウトアイコンに重ねるイメージ
            if(iconImage3 != null && layerGroupsLayerControl.Enabled)
            {
                if(layout != null)
                {
                    // カレント部分図レイアウト(部分図自体がカレントの場合も)
                    if(layout.IsCurrent|| layout.Drawing.IsCurrent)
                        e.Graphics.DrawImage(iconImage3, imageDrawRect2);// ペンマークを重ねる
                    // 表示のみ
                    else if(layout.Drawing.State == ShapeState.Visible)
                        e.Graphics.DrawImage(iconImage3, imageDrawRect3);// 鍵マークを重ねる
                }
            }

            // 文字の描画
            // 範囲
            TextFormatFlags textFormatFlags = TextFormatFlags.HorizontalCenter | TextFormatFlags.VerticalCenter | TextFormatFlags.NoPadding;// 中中に表示する、余白を省く（上下は残る）
            int drawTextSize = (int)(tabRect.Height * 0.55);
            int topLeftX = tabRect.X + (int)((tabRect.Width - drawTextSize)) - 4;
            int topLeftY = tabRect.Y + (int)((tabRect.Height - drawTextSize) * 0.5);
            Rectangle textDrawRect = new Rectangle(topLeftX, topLeftY, drawTextSize, drawTextSize);
            Font font = new Font(this.Font.FontFamily, textDrawRect.Height * 0.85f, GraphicsUnit.Pixel);

            string measureText = text;
            Size measureTextSize = TextRenderer.MeasureText(measureText, font, textDrawRect.Size, textFormatFlags);
            // 中央に描画するように位置調整
            int textOffsetX = (int)Math.Round((textDrawRect.Width - measureTextSize.Width) * 0.5, MidpointRounding.AwayFromZero);
            if(textOffsetX < 0)
                textOffsetX *= -1;

            int textOffsetY = (int)Math.Round((textDrawRect.Height - measureTextSize.Height) * 0.5, MidpointRounding.AwayFromZero);
            if(textOffsetY < 0)
                textOffsetY *= -1;

            Rectangle textDrawRect2 = new Rectangle(textDrawRect.X + textOffsetX, textDrawRect.Y - textOffsetY, textDrawRect.Width, textDrawRect.Height);

            // 文字描画            
			TextRenderer.DrawText(e.Graphics, text, font, textDrawRect2, textColor, textFormatFlags);
            font.Dispose();

            // 文字の周りの枠
            if(pen != null)
            {
                e.Graphics.DrawRectangle(pen, textDrawRect);
                pen.Dispose();
            }

            // 使われているレイヤグループのマーク
            if(isInUseLayerGroup)
            {
                Pen linePen = new Pen(Color.Orange, 3);
                Point p1 = new Point(textDrawRect.X, textDrawRect.Y - 2);
                Point p2 = new Point(textDrawRect.X + textDrawRect.Width / 2, p1.Y);
                e.Graphics.DrawLine(linePen, p1, p2);
                linePen.Dispose();
            }

            // 使われている部分図のマーク
            if(isShapesInDrawing)
            {
                Pen linePen = new Pen(Color.Green, 3);
                Point p1 = new Point(textDrawRect.X + textDrawRect.Width / 2, textDrawRect.Y - 2);
                Point p2 = new Point(textDrawRect.Right + 1, p1.Y);
                e.Graphics.DrawLine(linePen, p1, p2);
                linePen.Dispose();
            }

            // マウスがタブコントロールの領域内に入ったときのHotTrack描画(選択されていないタブの上にのみ)
            if(tabIndex != this.SelectedIndex)
            {
                Point sp = Cursor.Position;
                Point cp = this.PointToClient(sp);
                int tabIndexUnderMouse = GetTabIndexUnderPoint(cp);
                if(tabIndex == tabIndexUnderMouse && layerGroupsLayerControl.Enabled)
                {
                    Color baseColor = SystemColors.HotTrack;
                    SolidBrush hotTrackBrush = new SolidBrush(Color.FromArgb(128, baseColor));
                    e.Graphics.FillRectangle(hotTrackBrush, tabRect);

                    // マウスがタブコントロールの領域内に入ったときのタブのインデックス更新
                    onMouseTabIndex = tabIndex;

                    hotTrackBrush.Dispose();
                }
            }

            // 最後のタブの場合、タブ領域の隣からタブコントロール全体の端まで色を塗る
            if(tabIndex == this.TabPages.Count - 1)
            {
                System.Drawing.Color notTabAreaBackColor = tabPage.BackColor;
				System.Drawing.Brush notTabAreaBrush = new SolidBrush(notTabAreaBackColor);
                Rectangle lastTabRect = tabRect;
                int offsetX = 3;
                int offsetY = 2;
                e.Graphics.FillRectangle(notTabAreaBrush, lastTabRect.X - offsetX, lastTabRect.Bottom + offsetY, lastTabRect.Width + offsetX, this.Bottom - lastTabRect.Bottom - offsetY);

                notTabAreaBrush.Dispose();
			}

        }

        /// <summary>
        /// 指定したタブの LayerGroupsLayerControl 取得
        /// </summary>
        /// <param name="tabIndex"></param>
        /// <returns></returns>
        private LayerGroupsLayerControl GetLayerGroupsLayerControl(int tabIndex)
        {
            Debug.Assert(tabIndex >= 0 && tabIndex < this.TabCount);

            LayerGroupsLayerControl layerGroupsLayerControl = null;
            Control.ControlCollection controls = this.TabPages[tabIndex].Controls;
            foreach(Control control in controls)
            {
                if(control is LayerGroupsLayerControl)
                {
                    layerGroupsLayerControl = (LayerGroupsLayerControl)control;
                    break;
                }
            }

            return layerGroupsLayerControl;
        }

        /// <summary>
        /// 指定したPointがどのタブの上か取得
        /// </summary>
        /// <param name="point"></param>
        /// <returns></returns>
        public int GetTabIndexUnderPoint(Point point)
        {
            int tabIndex = -1;
            int tabCount = this.TabCount;
            for(int i = 0; i < tabCount; i++)
            {
                Rectangle rect = this.GetTabRect(i);
                if(rect.Contains(point))
                {
                    tabIndex = i;
                    break;
                }
            }

            return tabIndex;
        }


    }
}
